"""
Connect to the Database
The first thing to do when working with a SQLite database
(and most other Python database libraries) is to
create a connection to it.
Any queries and operations are performed
using the connection,
which is closed after the work is finished.
"""

import sqlite3

import click
from flask import current_app, g
from flask.cli import with_appcontext


def get_db():
    if 'db' not in g:
        g.db = sqlite3.connect(
            current_app.config['DATABASE'],
            detect_types=sqlite3.PARSE_DECLTYPES
        )
        g.db.row_factory = sqlite3.Row

    return g.db


def close_db(e=None):
    db = g.pop('db', None)

    if db is not None:
        db.close()
        
# Add the Python functions that will run these SQL commands to the db.py file:


def init_db():
    db = get_db()

    with current_app.open_resource('schema.sql') as f:
        db.executescript(f.read().decode('utf8'))


@click.command('init-db')
@with_appcontext
def init_db_command():
    """Clear the existing data and create new tables."""
    init_db()
    click.echo('Initialized the database.')


def init_app(app):
    app.teardown_appcontext(close_db)
    app.cli.add_command(init_db_command)
    



# Copyright (c) 2021, 2022 Narotechnology, nash@narotechnology.com
# SPDX-License-Identifier: LicenseRef-Narotechnology-Proprietary

import logging
import os
import sys
from flaskr.hw.hw_base import HardwareBase


lg = logging.getLogger('hardware')
lg.propagate = 0
lg.info('=========================')
lg.info('importing uart')
lg.info('=========================')


class UART(HardwareBase):
    """My Class"""

    def __init__(self):
        super().__init__()
        lg.info('init UART start')
        self.attached = False  # nash add, 여기까지는 윈도즈에서도 가능
        if sys.platform == 'linux':
            self.attached = True
        else:
            lg.error('UART is *currently* only for linux')

        lg.info('init UART end')

    def hello(self):
        print('hello, from UART')
        if self.attached:
            print('UART attached')
        else:
            print('UART not attached')


uart = UART()



# Copyright (c) 2021, 2022 Narotechnology, nash@narotechnology.com
# SPDX-License-Identifier: LicenseRef-Narotechnology-Proprietary

import logging
import sys
import time
from flaskr.hw.hw_base import HardwareBase

lg = logging.getLogger('hardware')
lg.propagate = 0
lg.info('=========================')
lg.info('importing bme280')
lg.info('=========================')


class BME280(HardwareBase):
    """My Class"""
    def __init__(self):
        super().__init__()
        lg.info('init BME280 start')
        self.attached = False  # nash add, 여기까지는 윈도즈에서도 가능
        if sys.platform == 'linux':
            import qwiic_bme280
            from flaskr.hw import i2c
            if 0x77 in i2c.scan():
                self.mySensor = qwiic_bme280.QwiicBme280()

                if self.mySensor.connected:
                    lg.info(HardwareBase.msgDevConnected)
                    msg = 'addr: ' + hex(self.mySensor.address)
                    lg.info(msg)
                    # print(msg)

                    self.mySensor.begin()
                    self.attached = True
                    time.sleep(1)  # give a sec for system messages to complete
                else:
                    print('bme280 not found')

            else:
                lg.info(HardwareBase.msgDevNotAttached)
                self.attached = False
        else:
            lg.info(HardwareBase.msgNotSupportedOS)
            # lg.error('bme280 is only for linux')

        lg.info('init BME280 end')

    def sensor_test(self):
        if self.attached:
            print("Humidity:\t%.3f" % self.mySensor.humidity)
        else:
            msg = 'Not found BME280'
            print(msg)
            lg.error(msg)

    def measure_temp(self):
        if self.attached:
            ret = self.mySensor.temperature_celsius     # 21.1203, 라이브러리에 문제가 있다.
        else:
            ret = -36.5
        return ret

    def measure_humi(self):
        if self.attached:
            ret = self.mySensor.humidity
        else:
            ret = -55.5
        return ret

    def measure_pres(self):
        if self.attached:
            ret = self.mySensor.pressure
        else:
            ret = -1013
        return ret

    def measure_alti(self):
        if self.attached:
            ret = self.mySensor.altitude_meters
        else:
            ret = -2744
        return ret


bme280 = BME280()
# bme280.sensor_test()




# Copyright (c) 2021, 2022 Narotechnology, nash@narotechnology.com
# SPDX-License-Identifier: LicenseRef-Narotechnology-Proprietary

import logging
import sys
import random
import time

from flaskr.hw.hw_base import HardwareBase

lg = logging.getLogger('hardware')
lg.propagate = 0
lg.info('=========================')
lg.info('importing adc')
lg.info('=========================')


# class SensorADC:
class SensorADC(HardwareBase):
    """My Class"""

    def __init__(self, add=0x48):
        super().__init__()
        """https://github.com/adafruit/Adafruit_CircuitPython_ADS1x15
        https://circuitpython.readthedocs.io/projects/ads1x15/en/latest/api.html
        """

        # time.sleep(2)
        self.attached = False
        self.data_adc = []
        self.data_time = []
        msg = 'init SensorADC start: ' + hex(add)
        lg.info(msg)
        # print(msg + '====================')

        if sys.platform == 'linux':
            from flaskr.hw import i2c
            import adafruit_ads1x15.ads1015 as ADS
            from adafruit_ads1x15.analog_in import AnalogIn

            if add in i2c.scan():
                self.ads = ADS.ADS1015(i2c, data_rate=3300, mode=ADS.Mode.CONTINUOUS, address=0x48)
                self.channel = AnalogIn(self.ads, ADS.P3)
                # self.channel = AnalogIn(self.ads, ADS.P0, ADS.P1)
                self.attached = True
                msg = '  found i2c device' + hex(add)
                lg.info(msg)
                # print(msg)
            else:
                msg = '  not found i2c device'
                lg.info(msg)
                # print(msg)
        else:
            msg = HardwareBase.msgNotSupportedOS
            lg.info(msg)
            # print(msg)

        # msg = 'init SensorADC end: ' + hex(add)
        # lg.info(msg)
        # print(msg)

    def read(self):
        if self.attached:
            return self.channel.value
        else:
            fake_data = random.randint(-200, -100)
            return fake_data

    # @staticmethod
    def hello(self):
        print('hello, from SensorADC')
        if self.attached:
            print('ADC attached')
        else:
            print('ADC not attached')


time.sleep(2)   # wait for... flask server
ad1 = SensorADC(add=0x48)
ad2 = SensorADC(add=0x49)

# lg.info('adc imported')
# lg.info(ad1.name)



# Copyright (c) 2021, 2022 Narotechnology, nash@narotechnology.com
# SPDX-License-Identifier: LicenseRef-Narotechnology-Proprietary

"""
`ModuleName`
====================================================

module for the ...

* Author(s): NaSeokhwan (2021)
"""

import sys
import logging
import socket
import threading
import time
import json
import statistics
import requests
import naro

print('<-', __name__)

from flaskr import my_globals as gm
from flaskr import __about__
from flaskr.hw import adc
from flaskr.hw import lcd
from flaskr.hw import cpu
from flaskr.hw import uart
from flaskr.hw import bme280
from flaskr.hw import ledcolor as led

print(__about__.__title__, __about__.__version__, __about__.__build_date__)
print(__about__.__summary__)


# # 외부 파일 참조
# if sys.platform == 'win32':
#     sys.path.append('e://bb//python//serial-com')
# elif sys.platform == 'linux':
#     sys.path.append('/home/pi/mnt/python/serial-com')
# from wdn400s import ModemMqtt

from flaskr.hw.wdn400s import ModemMqtt

# 기본 로거 설정
mylog = logging.getLogger('mylog')

# 주변장치 존재 여부
gm.peri['adc'] = adc.ad1.attached
gm.peri['lcd1'] = lcd.disp1.attached
gm.peri['bme280'] = bme280.bme280.attached

# ThingsBoard server sample: 'http://nash.wo.tc:9090/api/v1/RASPBERRY_PI_DEMO_TOKEN/telemetry'
things_cred = socket.gethostname().lower()  # 'RPI-LASER01'

if 1:
    # things_sv1 = 'http://nash.wo.tc:9090/api/v1/'  # 내 사이트, 21.1211부터 종료함
    things_sv1 = 'http://nash.mywire.org:9090/api/v1/'  # 내 사이트
    things1 = things_sv1 + things_cred + '/telemetry'

if 0:
    things_sv2 = 'https://demo.thingsboard.io/api/v1/'  # 온라인 데모 사이트
    things2 = things_sv2 + things_cred + '/telemetry'

things_headers = {
    'Content-type': 'application/json',
}


def list_diff(lst):
    diff = []
    p_x = 0
    for x in lst:
        us = int((x - p_x) * 1000 * 1000)  # micro second
        diff.append(us)
        p_x = x
    diff[0] = 0
    return diff


def config_from_to_file(cmd):
    """앱 설정을 파일에 쓰거나 불러오기"""
    dic = gm.cnfg
    # print(__name__, 'dic:', dic)
    # lg.info(f'gm.cnfg: {gm.cnfg}')

    # dic --> json
    jo = json.dumps(dic, ensure_ascii=True)  # 한글금지
    # print(__name__, 'json:', jo)

    conf_filename = 'config_app.json'

    if cmd is 'save':
        # 'wt' 를 사용하면 더러워진다 그리고 숫자를 스트링 처리한다
        with open(conf_filename, 'w') as f:  # with: 파일닫기를 자동으로
            # json --> file
            json.dump(jo, f)  # indent 안먹힌다
            print(__name__, 'save config to file:', jo)

    if cmd is 'load':
        try:
            with open(conf_filename, 'r') as f:
                # file --> json
                jo = json.load(f)
                # print(__name__, 'load config from file:', jo)
                mylog.info('load config from file:' + str(jo))

                # json --> python dic
                dic = json.loads(jo)
                gm.cnfg = dic
                print(__name__, 'config file found')
                print(__name__, 'default global config values will be replaced!!!')
        except:
            print('no config file found')


# if True:
def create_threads():
    """create ny threads"""

    print(__name__, 'fun:create_threads() start')
    print(__name__, socket.gethostname() + ', ' + time.asctime())

    if True:    # config structure changed, 개발시, 첫 배치에는 항상 이걸로 하라.
        config_from_to_file(cmd='save')
    else:       # 배치 후 테스트 완료 후 이 모드를 사용해야 한다.
        config_from_to_file(cmd='load')
        gm.cnfg['from_file'] = True

    def th_func2(arg1):
        """ADC read 데이터를 다른 함수에서 사용가능하게 한다"""
        # print('fun_2')
        lg = logging.getLogger('th_func2')
        lg.propagate = 0
        lg.setLevel(logging.DEBUG)
        lg.critical('---')
        lg.info(time.asctime())
        lg.info(gm.cnfg)

        t1 = naro.Timer()
        t1.clear()

        # print('\n\n=======Test===', gm.cnfg)
        # print('\n\n=======Test===', gm.lte2)

        cnt = 0
        while True:
            cnt += 1
            lst_time = []
            lst_data = []
            size = gm.cnfg['sample']['size']
            period = float(gm.cnfg['sample']['period']) / 1000  # ms -> second
            MIN_PERIOD_ADS = 1 / 3300  # for ADS1015. 0.303ms
            MIN_PERIOD = 1 / 4000
            decimation = int(period / MIN_PERIOD)
            assert decimation >= 1
            gm.status['sample']['decimation'] = decimation

            if cnt < gm.max_log:
                lg.info(f'cnt: {cnt}, decimation: {decimation}')
                if period < MIN_PERIOD_ADS:
                    lg.info('sampling period setting is too low. 0.33ms will be used instead.')

            if period < 1.0 / 1000:  # high speed, Decimation, 모두 이걸로 테스트
                gm.status['sample']['mode'] = 'Decimation'
                gm.status['sample']['delay'] = None
                for j in range(size * decimation):
                    distance = adc.ad1.read()
                    now = time.time()
                    if j % decimation == 0:
                        lst_data.append(distance)
                        lst_time.append(now)
                        # print('adc capture: ', j, lst_data)
            elif period < 10.0 / 1000:  # mid speed, Delay
                gm.status['sample']['mode'] = 'Delay'
                gm.status['sample']['decimation'] = None
                gm.status['sample']['delay'] = period - 0.20 / 1000  # for mid speed
                for j in range(size):
                    '''대책없이 기다린다. 현재로는 이게 가장 낫다.'''
                    t1.delay_sec(gm.status['sample']['delay'])
                    distance = adc.ad1.read()
                    now = time.time()
                    lst_data.append(distance)
                    lst_time.append(now)
                pass
            else:  # low speed, 이 경우는 현재 없다
                assert period < 10.0 / 1000

                for j in range(size):
                    time.sleep(period)
                    distance = adc.ad1.read()
                    now = time.time()
                    lst_data.append(distance)
                    lst_time.append(now)
                pass

            # 데이터 필터링

            gm.adc_data_mean = statistics.mean(lst_data)
            lst_time_diff = list_diff(lst_time)

            # 로그 파일에 기록

            if cnt < gm.max_log:
                lg.info(lst_time)
                lg.info(lst_time_diff)
                lg.info(lst_data)
                tmp = gm.adc_data_mean
                lg.info(f'adc data_mean: {tmp}')

                # 임시 콘솔에 표시
                # print(__name__, 'ADC:', lst_data)
                # print(__name__, 'TimeDiff:', lst_time_diff)
            elif cnt == gm.max_log:
                lg.info('log count reached max_log(fun2)')
            else:
                pass

            # 웹서버를 위한 데이터 복사 일단 no deep copy
            # print('gm.adc.data, lst_data:', gm.adc_data, lst_data)
            gm.adc_time = lst_time
            gm.adc_data = lst_data
            gm.adc_time_diff = lst_time_diff

            # ThingsBoard
            if cnt % 5 == 0:
                gm.tele1_main['displacement'] = gm.adc_data_mean
                data = json.dumps(gm.tele1_main)

                try:
                    response = requests.post(things1, headers=things_headers, data=data)
                except:
                    lg.error('network error(fun2), Thingsboard server')
                    pass

                # print(things1)

            time.sleep(0.2)  # 200ms:
            # time.sleep(5)     # 초당 5~10 회가 적당

    # th = threading.Thread(target=th_func2_adc_max, args=(1,))
    th = threading.Thread(target=th_func2, daemon=True, args=(1,))
    th.start()
    # th.join() # 세련된 방법
    '''위 두가지 모두 무한루프 쓰레드를 종료시키지 못하는 듯 21.08
    해결:
        데몬 설정이 먹힌다 그런데 start() 전에 설정하니 성공
    '''

    def th_fun3_100ms(arg1):
        cnt = 0
        while True:
            cnt += 1
            if cnt < gm.max_log:
                pass
            elif cnt == gm.max_log:
                pass
            else:
                pass
            time.sleep(0.1)
            pass

    th3 = threading.Thread(target=th_fun3_100ms, daemon=True, args=(2,))
    th3.start()

    def th_fun4_cpu_temp(arg1):
        cnt = 0
        while True:
            cnt += 1
            gm.status['cpu_temp'] = cpu.cpu.get_cpu_temp()

            # if cnt < gm.max_log:
            #     mylog.debug('cpu temp: ' + str(gm.status['cpu_temp']))
            # elif cnt == gm.max_log:
            #     mylog.debug('log count reached max_log(fun4)')
            #     mylog.debug(str(gm.peri))
            # else:
            #     pass

            time.sleep(2)

    th4 = threading.Thread(target=th_fun4_cpu_temp, daemon=True, args=(2,))
    th4.start()

    def th_fun5_lcd(arg1):
        time_fmt = '%m%d,%H:%M:%S'
        s22 = time.strftime(time_fmt, time.localtime(time.time()))
        lcd.disp1.set_cursor(0, 0)
        lcd.disp1.print(s22)

        cnt = 0
        while True:
            cnt += 1
            lcd.disp1.set_cursor(0, 16 * 1)
            lcd.disp1.print(f'[{cnt}]')
            time.sleep(1.0)

    th5 = threading.Thread(target=th_fun5_lcd, daemon=True, args=(2,))
    th5.start()

    def th_fun6_serial(arg1):
        cnt = 0
        while True:
            cnt += 1
            time.sleep(0.5)
            pass

    th6 = threading.Thread(target=th_fun6_serial, daemon=True, args=(2,))
    th6.start()

    def th_fun7_bme280(arg1):
        time.sleep(2)
        cnt = 0
        while True:
            gm.env['temp'] = bme280.bme280.measure_temp()  # Rpi OS 21.10 에서는 라이브러리 버그,
            gm.env['humi'] = bme280.bme280.measure_humi()
            gm.env['pres'] = bme280.bme280.measure_pres()
            gm.env['alti'] = bme280.bme280.measure_alti()

            gm.tele2_env['temperature'] = gm.env['temp']
            gm.tele2_env['humidity'] = gm.env['humi']
            gm.tele2_env['pressure'] = gm.env['pres']
            gm.tele2_env['cpu_temperature'] = gm.status['cpu_temp']

            # if cnt < 3:
            #     mylog.debug('fun7_bme280 test: ' + str(gm.env))
            # elif cnt == 3:
            #     pass
            # else:
            #     pass

            cnt += 1
            time.sleep(1)
            pass

    th7 = threading.Thread(target=th_fun7_bme280, daemon=True, args=(2,))
    th7.start()

    def th_fun8_iot_server(arg1):
        """publish to MQTT Broker"""
        cnt = 0
        while True:
            data = json.dumps(gm.tele2_env)
            try:
                response = requests.post(things1, headers=things_headers, data=data)
            except:
                mylog.error('things1 request fail')

            cnt += 1
            time.sleep(2)

    th8 = threading.Thread(target=th_fun8_iot_server, daemon=True, args=(2,))
    th8.start()

    def th_fun9(arg1):
        """change neopixel led color"""

        gm.status['thread']['led'] = True

        cnt = 0
        while True:
            # mylog.info('ledcolor running...')
            # ledcolor.ledcolor.hello()

            if cnt % 3 == 0:
                led.ledcolor.color_green()
            elif cnt % 3 == 1:
                led.ledcolor.color_red()
            else:
                led.ledcolor.color_blue()

            cnt += 1
            time.sleep(1)

    th9 = threading.Thread(target=th_fun9, daemon=True, args=(2,))
    th9.start()

    def th_fun12(arg1):
        """lte-m1 modem mqtt"""

        def print2(*s):
            if True:
                print(__name__, *s)

        gm.status['thread']['lte'] = True

        # dev1 = '/dev/ttyUSB0'
        # dev1 = 'COM16'
        if sys.platform == 'win32':
            dev1 = 'COM16'
        elif sys.platform == 'linux':
            dev1 = '/dev/ttyUSB1'
        elif sys.platform == 'cygwin':
            pass
        else:
            pass

        baud_rate = gm.cnfg['lte']['baud_rate']     # 115200  # 19200 # 9600
        time_out = gm.cnfg['lte']['time_out']

        print('modem opening...')
        modem = ModemMqtt()  # dev1, baudrate=baud_rate, timeout=time_out)
        print('config...')
        at_cmds = [
            'AT*WMQTCFG=endpoint,mqtt.gleeze.com',
            'AT*WMQTCFG=username,forall'
        ]
        modem.at_config(at_cmds, wait=0.5)  # 모뎀 부팅 후 한번만
        s1 = '''AT*WMQTPUB='''
        s2 = '''v1/devices/me/telemetry,'''

        cnt = 0
        while True:
            cnt += 1
            print('connecting...')
            modem.at_connect()  # 전송 전

            dic1 = {'value': cnt, 'host': gm.hostname + '-wdn400s'}
            s3 = '\"' + str(dic1) + '\"'
            cmd = s1 + s2 + s3
            print2(cmd)

            print('sending...')
            modem.at_mqtt_pub(cmd)

            print('disconnecting...')
            modem.at_disconnect()  # 전송 후

            # print('modem closing...')
            # modem.close()  # 프로그램 완전 종료 시에만 호출하라

            if cnt < 2:  # 처음에만 자주 전송한다.
                time.sleep(10)
            elif cnt == 2:
                print('next time, lte send_interval will be change to:', gm.cnfg['lte']['send_interval'])
            else:
                time.sleep(gm.cnfg['lte']['send_interval'])

    th12 = threading.Thread(target=th_fun12, daemon=True, args=(2,))
    th12.start()




#-*- coding:utf-8 -*-
# Copyright (c) 2021, 2022 Narotechnology, nash@narotechnology.com
# SPDX-License-Identifier: LicenseRef-Narotechnology-Proprietary

"""

"""
import time

print('<-', __name__)


__title__ = "naro-laser"
__summary__ = "Laser Sensor Backend App"
__version__ = '0.1.2'
__build_date__ = '2022.01.11'
__uri__ = "https://test.pypi.org/"

'''
    Korean not allowed currently, 21.12
    https://github.com/pypa/warehouse/blob/64ca42e42d5613c8339b3ec5e1cb7765c6b23083/warehouse/__about__.py
'''

'''
Release History 

0.1.2, 22.01.11
    
0.1.2rc3, 22.01.11
    repair
    
0.1.2rc2, 22.01.11
    add mini web server
    add lte-m1 control    
    
0.1.1, 21.12.23
    auto start using crontab
    change execution mode to 'sudo pipenv...'
    
0.1.1rc2, 21.12.22
    add color led neopixel control routine
        sudo required
    change thingsboard url:
        nash.wo.tc -> nash.mywire.org

'''


#  Copyright (c) 2021. Lorem ipsum dolor sit amet, consectetur adipiscing elit.
#  Morbi non lorem porttitor neque feugiat blandit. Ut vitae ipsum eget quam lacinia accumsan.
#  Etiam sed turpis ac ipsum condimentum fringilla. Maecenas magna.
#  Proin dapibus sapien vel ante. Aliquam erat volutpat. Pellentesque sagittis ligula eget metus.
#  Vestibulum commodo. Ut rhoncus gravida arcu.



"""하나의 블루프린트 인듯"""
from flask import (
    Blueprint, flash, g, redirect,
    render_template, request, url_for
)
from werkzeug.exceptions import abort

from flaskr.auth import login_required
from flaskr.db import get_db

bp = Blueprint('blog', __name__)


def get_post(id, check_author=True):
    post = get_db().execute(
        'SELECT p.id, title, body, created, author_id, username'
        ' FROM post p JOIN user u ON p.author_id = u.id'
        ' WHERE p.id = ?',
        (id,)
    ).fetchone()

    if post is None:
        abort(404, f"Post id {id} doesn't exist.")

    if check_author and post['author_id'] != g.user['id']:
        abort(403)

    return post


# @bp.route('/') # 루트에서 제거한다. 21.10
@bp.route('/blog')
def index():
    db = get_db()
    posts = db.execute(
        'SELECT p.id, title, body, created, author_id, username'
        ' FROM post p JOIN user u ON p.author_id = u.id'
        ' ORDER BY created DESC'
    ).fetchall()
    return render_template('blog/index.html', posts=posts)


@bp.route('/create', methods=('GET', 'POST'))
@login_required
def create():
    if request.method == 'POST':
        title = request.form['title']
        body = request.form['body']
        error = None

        if not title:
            error = 'Title is required.'

        if error is not None:
            flash(error)
        else:
            db = get_db()
            db.execute(
                'INSERT INTO post (title, body, author_id)'
                ' VALUES (?, ?, ?)',
                (title, body, g.user['id'])
            )
            db.commit()
            return redirect(url_for('blog.index'))

    return render_template('blog/create.html')


@bp.route('/<int:id>/update', methods=('GET', 'POST'))
@login_required
def update(id):
    post = get_post(id)

    if request.method == 'POST':
        title = request.form['title']
        body = request.form['body']
        error = None

        if not title:
            error = 'Title is required.'

        if error is not None:
            flash(error)
        else:
            db = get_db()
            db.execute(
                'UPDATE post SET title = ?, body = ?'
                ' WHERE id = ?',
                (title, body, id)
            )
            db.commit()
            return redirect(url_for('blog.index'))

    return render_template('blog/update.html', post=post)


@bp.route('/<int:id>/delete', methods=('POST',))
@login_required
def delete(id):
    get_post(id)
    db = get_db()
    db.execute('DELETE FROM post WHERE id = ?', (id,))
    db.commit()
    return redirect(url_for('blog.index'))



# Copyright (c) 2021, 2022 Narotechnology, nash@narotechnology.com
# SPDX-License-Identifier: LicenseRef-Narotechnology-Proprietary

import logging
import sys
import time

import qwiic_serlcd
import qwiic_micro_oled
import qwiic_oled_display

from flaskr.hw.hw_base import HardwareBase

lg = logging.getLogger('hardware')
lg.propagate = 0
lg.info('=========================')
lg.info('importing lcd')
lg.info('=========================')


class OLED1(HardwareBase):
    """128x48(?)"""
    def __init__(self):
        super().__init__()
        lg.info('init class OLED')
        self.attached = False  # nash add, 여기까지는 윈도즈에서도 가능
        if sys.platform == 'linux':
            if False:
                try:
                    lg.info('finding OLED...(try..except)')
                    print('=== trying...')
                    self._myDisp = qwiic_oled_display.QwiicOledDisplay()
                    # self._myDisp = qwiic_micro_oled.QwiicMicroOled()
                    print('===', self._myDisp.connected)
                except:
                    lg.info(HardwareBase.msgDevNotAttached)
                    print('Not found display')

            self._myDisp = qwiic_oled_display.QwiicOledDisplay()
            # print('self._myDisp connected=', self._myDisp.connected)

            if self._myDisp.connected:
                self._myDisp.begin()
                self._myDisp.clear(self._myDisp.ALL)
                self._myDisp.clear(self._myDisp.PAGE)
                self._myDisp.set_font_type(1)

                self.attached = True
                lg.info(HardwareBase.msgDevConnected)
            else:
                self.attached = False
                lg.info(HardwareBase.msgDevNotAttached)
        else:
            lg.info(HardwareBase.msgNotSupportedOS)

    def print(self, s):
        if self.attached:
            self._myDisp.print(s)
            self._myDisp.display()

    def set_cursor(self, col, row):
        if self.attached:
            self._myDisp.set_cursor(col, row)

    def clear(self):
        if self.attached:
            self._myDisp.clear(self._myDisp.ALL)
            # self._myDisp.clear(self._myDisp.PAGE)


disp1 = OLED1()

disp1.print('Disp1')
time.sleep(1)
disp1.clear()



# Copyright (c) 2021, 2022 Narotechnology, nash@narotechnology.com
# SPDX-License-Identifier: LicenseRef-Narotechnology-Proprietary

import logging
import os
import sys

from flaskr.hw.hw_base import HardwareBase


lg = logging.getLogger('hardware')
lg.propagate = 0
lg.info('=========================')
lg.info('importing ledcolor')
lg.info('=========================')


class LEDColor(HardwareBase):
    """My Class"""

    def __init__(self):
        super().__init__()
        lg.info('init LEDColor start')
        self.attached = False  # nash add, 여기까지는 윈도즈에서도 가능
        if sys.platform == 'linux':
            import board
            import neopixel

            self.pin_out = board.D21    # PWM
            self.pixels = neopixel.NeoPixel(self.pin_out, 1)
            self.pixels[0] = (5, 0, 0)
            self.pixels.show()

            self.attached = True
        else:
            lg.error('ledcolor is only for linux')

        lg.info('init LEDColor end')

    def hello(self):
        if self.attached:
            print('hello from ledcolor 1')
        else:
            print('hello from ledcolor 2')

    def color_red(self):
        print('RED  ', end='\r')  # use waitress, not flask!
        if self.attached:
            self.pixels[0] = (0, 5, 0)
            self.pixels.show()

    def color_green(self):
        print('GREEN ', end='\r')
        if self.attached:
            self.pixels[0] = (5, 0, 0)
            self.pixels.show()

    def color_blue(self):
        print('BLUE ', end='\r')
        if self.attached:
            self.pixels[0] = (0, 0, 5)
            self.pixels.show()


ledcolor = LEDColor()



# Copyright (c) 2021, 2022 Narotechnology, nash@narotechnology.com
# SPDX-License-Identifier: LicenseRef-Narotechnology-Proprietary
'''
TabbedPanel
============

Test of the widget TabbedPanel.
'''

from kivy.app import App
from kivy.uix.tabbedpanel import TabbedPanel
from kivy.lang import Builder
from kivy.core.window import Window

import my_globals as g

Builder.load_string("""
#:import g my_globals
<Test>:
    # size_hint: .5, .5
    pos_hint: {'center_x': .5, 'center_y': .5}
    do_default_tab: False

    TabbedPanelItem:
        text: 'Commands'
        BoxLayout:
            orientation: 'vertical'
            Button:
                text: 'Save'
                id: bt_save
                on_press: 
                    print('Save pressed')
                    g.config['sample']['period'] = float(spin_sample_period.text)
                    g.config['sample']['size'] = int(spin_sample_size.text)
                    g.config['log']['max'] = int(spin_log_max.text)
                    g.config['proc']['method'] = spin_proc_method.text
                    print(g.config)                    
                    root.on_config_file('save')               
            Button:
                text: 'Load'
                on_press: 
                    print('Load pressed')
                    root.on_config_file('load')               
            Button:
                text: 'Close'
                on_press: 
                    print('Close pressed')
                                    
            Button:
                text: 'Test'
                on_press: 
                    self.text = self.text + '@'
                    bt_save.text = '***'
                    g.config['sample']['period'] = 100
                    print(g.config)
                    print('not implemented ****')
            Button:
                text: 'Blank'
            Button:
                text: 'Blank'
            Button:
                text: 'Blank'
                
                
    TabbedPanelItem:
        text: 'Config'
        BoxLayout:
            orientation: 'vertical'

            Label:
                text: 'Sample Period(ms):'
            Spinner:
                id: spin_sample_period
                text: '1.0'
                values: '0.25', '0.5', '1.0', '2.0', '5.0'

            Label:
                text: 'Sample Size:'
            Spinner:
                id: spin_sample_size
                text: '8'
                values: '4', '8', '16', '32'

            Label:
                text: 'Max # of logs:'
            Spinner:
                id: spin_log_max
                text: '10'
                values: '20', '50', '100', '1000'

            Label:
                text: 'Post processing:'
            Spinner:
                id: spin_proc_method
                text: 'MEAN'
                values: 'MEAN', 'MEDIAN'

            Label:
                text: 'Blank:'
            Spinner:
                id: spin_blank1
                text: '0'
                values: '0', '1', '2'
            Label:
                text: 'Blank:'
            Spinner:
                id: spin_blank2
                text: '0'
                values: '0', '1', '2'

                
    TabbedPanelItem:
        text: 'Help'
        RstDocument:
            text:
                '\\n'.join(("Hello world", "-----------",
                "You are in the third tab."))

""")


class Test(TabbedPanel):
    def on_config_file(self, cmd):
        dic = g.config
        # print(__name__, 'dic:', dic)
        jo = json.dumps(dic, ensure_ascii=True)  # 한글금지
        # print(__name__, 'json:', jo)

        conf_filename = 'config_app.json'

        # if cmd is 'save':
        #     with open(conf_filename, 'w') as f:
        #         json.dump(jo, f, indent=4)  # indent 안먹힌다
        #         print(__name__, 'config json to file:', jo)
        #
        # if cmd is 'load':
        #     with open(conf_filename, 'r') as f:
        #         jo = json.load(f)
        #         print(__name__, 'config json from file:', jo)

        if cmd is 'save':
            # 'wt' 를 사용하면 더러워진다 그리고 숫자를 스트링 처리한다
            with open(conf_filename, 'w') as f:
                # json --> file
                json.dump(jo, f)  # indent 안먹힌다
                print(__name__, 'save config to file:', jo)

        if cmd is 'load':
            with open(conf_filename, 'r') as f:
                # file --> json
                jo = json.load(f)
                print(__name__, 'load config from file:', jo)
                # json --> python dic
                dic = json.loads(jo)
                g.config = dic
                print(g)


    # def foo1(self):
    #     print('foo...')
    #     self.config_file(cmd='save')

    # def on_config_file(self, cmd):

    pass


class TabbedPanelApp(App):
    def build(self):
        return Test()


import json

if __name__ == '__main__':
    app = TabbedPanelApp()
    # Window.size = (320, 480)  # good TFT, 3.5"
    Window.size = (320, 400)  # good TFT, 3.5"
    Window.top = 0
    Window.left = 0
    app.run()



# Copyright (c) 2021, 2022 Narotechnology, nash@narotechnology.com
# SPDX-License-Identifier: LicenseRef-Narotechnology-Proprietary

import logging
import sys


if sys.platform == 'linux':
    from flaskr.hw import i2c
    import adafruit_ads1x15.ads1015 as ADS
    from adafruit_ads1x15.analog_in import AnalogIn


class SensorADC:
    """My Class"""
    def __init__(self, addr=0x48):
        """https://github.com/adafruit/Adafruit_CircuitPython_ADS1x15
        https://circuitpython.readthedocs.io/projects/ads1x15/en/latest/api.html
        https://github.com/adafruit/Adafruit_CircuitPython_MCP230xx
        """
        self.attached = False
        self.data_adc = []
        self.data_time = []
        logging.warning('init SensorADC start')
        if sys.platform == 'linux':
            if addr in i2c.scan():
                self.ads = ADS.ADS1015(i2c, data_rate=3300, mode=ADS.Mode.CONTINUOUS, address=0x48)
                self.channel = AnalogIn(self.ads, ADS.P3)
                # self.channel = AnalogIn(self.ads, ADS.P0, ADS.P1)
                self.attached = True
                logging.warning('Found i2c device' + hex(addr))
            else:
                logging.warning('Not found i2c device' + hex(addr))
        else:
            logging.warning('Only for linux')

        logging.warning('init SensorADC end')



# Copyright (c) 2021, 2022 Narotechnology, nash@narotechnology.com
# SPDX-License-Identifier: LicenseRef-Narotechnology-Proprietary

"""
`wdn400s`
====================================================

module for the LTE-M1 comm.

* Author(s): NaSeokhwan (2021)
"""

import time
import serial
import serial.tools.list_ports


def print2(*s):
    if True:
        print(*s)


def print3(*s):
    if True:
        print2(*s)


class WDN400S:
    """WD-N400S Modem"""
    def __init__(self):
        print(__name__, 'init modem instance...')

        url = self.find_modem_wdn400s()

        if url:
            self.ser = serial.serial_for_url(url=url, baudrate=115200, timeout=1)
            self.attached = True
        else:
            print('** modem not attached')
            self.attached = False

    def find_modem_wdn400s(self):
        """find WD-N400S LTE M1 modem"""
        ports = serial.tools.list_ports.comports(include_links=not True)
        for p in ports:
            print(p.device, p.description)

            if 'CP2105' in p.description:
                if '/dev/ttyUSB1' in p.device:  # raspberry
                    print('- target modem found:', p.device)
                    return p.device
                if 'Standard' in p.description:  # win 10
                    print('- target modem found:', p.device)
                    return p.device
        return False

    def close(self):
        if self.attached:
            self.ser.close()

    def readable(self):
        if self.attached:
            return self.ser.readable()

    def read_all(self):
        if self.attached:
            return self.ser.read_all()

    def write(self, cmd_bytes):
        if self.attached:
            self.ser.write(cmd_bytes)

    def write_str_cr(self, cmd_str):
        if self.attached:
            self.ser.write((cmd_str+'\r').encode('utf-8'))

    def write_read(self, cmd, wait, err_str='xxx', ok_str='xxx'):
        if not self.attached:
            return
        print2('W:', cmd)
        self.write_str_cr(cmd)
        time.sleep(wait)
        b1 = self.read_all()
        print2('R:', b1)
        if err_str.encode('utf-8') in b1:
            print3('  --not ok')
        else:
            if ok_str.encode('utf-8') in b1:
                print3('  --ok')
            else:
                print3('  --not ok')

    def at_config(self, at_cmds, wait):
        for cmd in at_cmds:
            self.write_read(cmd, wait, ok_str='OK')


class ModemMqtt(WDN400S):
    def at_connect(self):
        cmd = 'AT*WMQTCON'
        super().write_read(cmd, 2.0, ok_str='WMQTCON:2')

    def at_disconnect(self):
        cmd = 'AT*WMQTDIS'
        super().write_read(cmd, 2.0, ok_str='WMQTDIS:0')

    def at_mqtt_pub(self, cmd,):
        super().write_read(cmd, 0.2, err_str='WMQTPUB:NC', ok_str='WMQTPUB:')


class ModemTcp(WDN400S):
    def at_connect(self):
        cmd = 'AT+WSOCO=0'
        super().write_read(cmd, 0.5, ok_str='OPEN_CMPL')
        # 'WSOCO:0' 로는 감지 불가, 중복

    def at_disconnect(self):
        cmd = 'AT+WSOCL=0'
        super().write_read(cmd, 0.5, ok_str='CLOSE_CMPL')

    def at_tcp_send(self, cmd):
        super().write_read(cmd, 0.1, ok_str='WSOWR:1')


class ModemHttp(WDN400S):
    def at_connect(self):
        cmd = 'AT*WHTTP=3'  # 페이지 로드, 많은 데이터, 사용 주의
        super().write_read(cmd, 10, ok_str='WHTTPR:COMPLETED')

    # def at_disconnect(self):
    #     cmd = 'AT+?WSOCL=0'
    #     super().write_read(cmd, 0.1, ok_str='?CLOSE_CMPL')

    def at_http_send(self, cmd):
        super().write_read(cmd, 0.1)


def demo_wo_class():
    """demo without class"""
    ser = serial.serial_for_url('COM16', baudrate=115200, timeout=1)

    at_cmds = [b'ati\r',
               b'AT*WMQTCFG=endpoint,nash.mywire.org\r']

    for cmd in at_cmds:
        print('-----')
        ser.write(cmd)
        time.sleep(1)
        if ser.readable():
            b1 = ser.read_all()
            print('-- read bytes:')
            print(b1)
            print('-- convert to string:')
            s1 = b1.decode('utf-8')
            # s1 = b1.decode('ascii')
            print(s1)   # pycharm, run 창에서는 보이지 않는 버그 발견.
                        # 반드시 실행창에서 실행하라.

            # s2 = s1.replace('\r', '').split('\n')
            # for line in s23:
            #     print(line)

            print('---')
        else:
            print('not readable')

    ser.close()


def demo_class():
    print('modem opening...')
    # modem = ModemMqtt('COM16', baudrate=115200, timeout=1)
    modem = ModemMqtt()
    print('config...')
    at_cmds = [
        'AT*WMQTCFG=endpoint,mqtt.gleeze.com',
        'AT*WMQTCFG=username,forall'
    ]
    modem.at_config(at_cmds, wait=0.5)  # 모뎀 부팅 후 한번만


if __name__ == '__main__':
    demo_wo_class()
    demo_class()




# Copyright (c) 2021, 2022 Narotechnology, nash@narotechnology.com
# SPDX-License-Identifier: LicenseRef-Narotechnology-Proprietary

import logging
import os
import sys
from flaskr.hw.hw_base import HardwareBase


lg = logging.getLogger('hardware')
lg.propagate = 0
lg.info('=========================')
lg.info('importing cpu')
lg.info('=========================')


class CPU(HardwareBase):
    """My Class"""

    def __init__(self):
        super().__init__()
        lg.info('init CPU start')
        self.attached = False  # nash add, 여기까지는 윈도즈에서도 가능
        if sys.platform == 'linux':
            self.attached = True
        else:
            lg.error('cpu is only for linux')

        lg.info('init CPU end')

    def get_cpu_temp(self):
        if self.attached:
            temp = os.popen("vcgencmd measure_temp").readline()
            temp = temp.replace('temp=', '')
            temp = temp.replace("'C", "")
            # print('replaced:', temp)
            return float(temp)
        else:
            return -273.1   # fake


cpu = CPU()



# Copyright (c) 2021, 2022 Narotechnology, nash@narotechnology.com
# SPDX-License-Identifier: LicenseRef-Narotechnology-Proprietary

class HardwareBase:
    name = 'HardwareBase Class'
    msgNotSupportedOS = '**Not** Supported OS'
    msgDevNotAttached = 'Device **Not** Attached'
    msgDevConnected = 'Device Connected, OK'

    def __init__(self):
        self.name = 'HardwareBase instance'

    @classmethod
    def hello(cls):
        return cls.name

    @classmethod
    def hello_pr(cls):
        print(cls.name)

    @classmethod
    def hw(cls):
        print(cls.name)




# Copyright (c) 2021, 2022 Narotechnology, nash@narotechnology.com
# SPDX-License-Identifier: LicenseRef-Narotechnology-Proprietary

"""임포트 시 실행되어버리는 것을 방지하는게 나을 듯
그러나 당분간 이대로 두자 21.09
"""

import logging
import sys

lg = logging.getLogger('hardware')
lg.propagate = 0
lg.info('=========================')
lg.info('importing hw')
lg.info('=========================')

if sys.platform == 'linux':
    '''i2c bus instance creation'''
    import board
    import busio

    # Create the I2C bus
    # ADC 외에도 lcd 등에서도 사용되므로 반드시 여기에 있어야 한다
    i2c = busio.I2C(board.SCL, board.SDA, 400000) # ???

    lg.info('Linux system')
else:
    lg.error('** NOT ** Linux system')






import functools

from flask import (
    Blueprint, flash, g, redirect,
    render_template, request, session, url_for
)
from werkzeug.security import check_password_hash, generate_password_hash

from flaskr.db import get_db

bp = Blueprint('auth', __name__, url_prefix='/auth')


@bp.route('/register', methods=('GET', 'POST'))
def register():
    if request.method == 'POST':
        username = request.form['username']
        password = request.form['password']
        db = get_db()
        error = None

        if not username:
            error = 'Username is required.'
        elif not password:
            error = 'Password is required.'
        elif db.execute(
            'SELECT id FROM user WHERE username = ?', (username,)
        ).fetchone() is not None:
            error = f"User {username} is already registered."

        if error is None:
            db.execute(
                'INSERT INTO user (username, password) VALUES (?, ?)',
                (username, generate_password_hash(password))
            )
            db.commit()
            return redirect(url_for('auth.login'))

        flash(error)

    return render_template('auth/register.html')


@bp.route('/login', methods=('GET', 'POST'))
def login():
    if request.method == 'POST':
        username = request.form['username']
        password = request.form['password']
        db = get_db()
        error = None
        user = db.execute(
            'SELECT * FROM user WHERE username = ?', (username,)
        ).fetchone()

        if user is None:
            error = 'Incorrect username.'
        elif not check_password_hash(user['password'], password):
            error = 'Incorrect password.'

        if error is None:
            session.clear()
            session['user_id'] = user['id']
            return redirect(url_for('index'))

        flash(error)

    return render_template('auth/login.html')


@bp.before_app_request
def load_logged_in_user():
    user_id = session.get('user_id')

    if user_id is None:
        g.user = None
    else:
        g.user = get_db().execute(
            'SELECT * FROM user WHERE id = ?', (user_id,)
        ).fetchone()


@bp.route('/logout')
def logout():
    session.clear()
    return redirect(url_for('index'))


def login_required(view):
    @functools.wraps(view)
    def wrapped_view(**kwargs):
        if g.user is None:
            return redirect(url_for('auth.login'))

        return view(**kwargs)

    return wrapped_view
    




# Copyright (c) 2021, 2022 Narotechnology, nash@narotechnology.com
# SPDX-License-Identifier: LicenseRef-Narotechnology-Proprietary
"""설정
작업 기록
21.09, 로깅설정을 json 파일로부터 읽어들이는 방식으로 변경
21.1004,
    스트림 핸들러 부분: 설정파일로부터
    파일 핸들러 부분: 소스파일 내에서 작성하기로 변경
"""
print('<-', __name__)

import logging
import logging.config
import json
import socket
import time


# if True:
def init_logger_old():
    """파일 기반 방식이다. 그런데 불편하다. 제거하자. 21.11
    """
    # sys.path.append('flaskr') # testing...
    # print(sys.path)
    # with open('.//flaskr//log_config.json', 'rt') as f:    # pycharm: OK

    print('\n\n*** logging config start')
    with open('config_log.json', 'rt') as f:  # pkg???
        config = json.load(f)
        print('--- logging config load')
    logging.config.dictConfig(config)
    # pprint.pprint(config)
    print('*** logging config end')

    '''로그 파일 폴더 및 이름'''
    prefix = './/log//'
    # prefix += time.strftime('%Y.%m%d.%H%M-')
    # prefix += time.strftime('%d.%H%M-')
    prefix += time.strftime('00-')  # 임시로...
    prefix += socket.gethostname()

    sf1 = '%(name)s:%(levelname)s:%(message)s'
    sf2 = '%(asctime)s:%(name)s:%(levelname)s:%(message)s'

    lg = logging.getLogger()
    f_h = logging.FileHandler(prefix + '-root.log', 'w')
    f_h.setFormatter(logging.Formatter(sf2))
    lg.addHandler(f_h)

    lg = logging.getLogger('th_func2')
    f_h = logging.FileHandler(prefix + '-th_func2.log', 'w')
    f_h.setFormatter(logging.Formatter(sf1))
    lg.addHandler(f_h)

    lg = logging.getLogger('th_func3')
    f_h = logging.FileHandler(prefix + '-th_func3.log', 'w')
    f_h.setFormatter(logging.Formatter(sf1))
    lg.addHandler(f_h)

    lg = logging.getLogger('hardware')
    f_h = logging.FileHandler(prefix + '-hardware.log', 'w')
    f_h.setFormatter(logging.Formatter(sf1))
    lg.addHandler(f_h)

    # logger test
    '''아래 테스트 결과는 이상하다
    이모듈에서는 정상인데 
    다른 모듈에서 로거를 호출하면 파일로그는 정상, 스트림로그는 불량이 된다 21.10'''

    lg = logging.getLogger('root')
    # logger_test(lg)

    # lg = logging.getLogger('my')
    # logger_test(lg)

    # lg = logging.getLogger('th_func2')
    # logger_test(lg)

    # lg = logging.getLogger('th_func3')
    # logger_test(lg)
    #
    # lg = logging.getLogger('hardware')
    # logger_test(lg)


def init_logger():
    """
        첫 로그파일을 원하는 위치에 생성한다.
    """

    def logger_test(logger, msg=None):
        logger.critical('=== logger output test start ===')
        txt = 'Hello, sample text...!!!'
        logger.error(txt)
        logger.warning(txt)
        logger.info(txt)
        logger.debug(txt)
        logger.info(time.asctime())
        logger.critical('=== logger setup test end ===')

    '''로그 파일 폴더 및 이름'''
    prefix = './/log//'
    prefix += time.strftime('0000-')  # 임시로...
    prefix += socket.gethostname()

    sf1 = '%(name)s:%(levelname)s:%(message)s'
    sf2_time = '%(asctime)s:%(name)s:%(levelname)5s: %(message)s'

    lg = logging.getLogger('mylog')
    s_h = logging.StreamHandler()
    f_h = logging.FileHandler(prefix + '-mylog.log', 'w')
    s_h.setFormatter(logging.Formatter(sf2_time))
    f_h.setFormatter(logging.Formatter(sf2_time))
    # lg.addHandler(s_h)
    lg.addHandler(f_h)
    lg.setLevel(logging.DEBUG)

    lg = logging.getLogger('th_func2')
    s_h = logging.StreamHandler()
    f_h = logging.FileHandler(prefix + '-th_func2.log', 'w')
    s_h.setFormatter(logging.Formatter(sf2_time))
    f_h.setFormatter(logging.Formatter(sf2_time))
    # lg.addHandler(s_h)    # 로그데이터이므로 화면출력에서는 제외
    lg.addHandler(f_h)
    lg.setLevel(logging.DEBUG)

    '''
    lg = logging.getLogger('th_func3')
    s_h = logging.StreamHandler()
    f_h = logging.FileHandler(prefix + '-th_func3.log', 'w')
    s_h.setFormatter(logging.Formatter(sf2_time))
    f_h.setFormatter(logging.Formatter(sf2_time))
    # lg.addHandler(s_h)    #
    lg.addHandler(f_h)
    lg.setLevel(logging.DEBUG)'''

    lg = logging.getLogger('hardware')
    s_h = logging.StreamHandler()
    f_h = logging.FileHandler(prefix + '-hardware.log', 'w')
    s_h.setFormatter(logging.Formatter(sf2_time))
    f_h.setFormatter(logging.Formatter(sf2_time))
    # lg.addHandler(s_h)    # 문제가 있으면 로그파일에서 확인하자
    lg.addHandler(f_h)
    lg.setLevel(logging.DEBUG)

    # logger test
    '''
    아래 테스트 결과는 이상하다
    이모듈에서는 정상인데 
    다른 모듈에서 로거를 호출하면 파일로그는 정상, 스트림로그는 불량이 된다 21.10
    한곳에서 스트림핸들러 파일핸들러 추가하고 레벨 정하니 된다 21.11
    '''
    if not True:
        lg = logging.getLogger('mylog')
        logger_test(lg)

        lg = logging.getLogger('th_func2')
        logger_test(lg)

        lg = logging.getLogger('th_func3')
        logger_test(lg)

        lg = logging.getLogger('hardware')
        logger_test(lg)


'''전략이 자주 왔다갔다 하고있다'''

init_logger()



# Copyright (c) 2021, 2022 Narotechnology, nash@narotechnology.com
# SPDX-License-Identifier: LicenseRef-Narotechnology-Proprietary
"""bp_navbar.py"""
import json
import random
import time
from datetime import datetime
from flask import (
    Blueprint, flash, g, redirect,
    render_template, request, url_for, Response
)
from werkzeug.exceptions import abort

from flaskr.auth import login_required
from flaskr.db import get_db
from flaskr import my_globals as g  # 22.0107 충돌 아냐?
from flaskr import __about__

import logging

lg = logging.getLogger('mylog')
lg.setLevel(logging.DEBUG)
lg.info('logger test, ' + time.asctime())

# lg = logging.getLogger('th_func2')
# lg.propagate = 0
# lg.setLevel(logging.DEBUG)
# lg.critical('---')
# lg.info(time.asctime())

bp = Blueprint('navbar', __name__)


@bp.route('/')  # __init__.py 도 참고하라.
@bp.route('/navbar')
@bp.route('/navbar/index')
def index():
    return render_template(
        '/navbar/index.html',
        title='My Navbar Index',
        message=g.hostname)


@bp.route('/navbar/status')
def status():
    about = [__about__.__title__, __about__.__build_date__, __about__.__version__]

    s1 = render_template(
        '/navbar/status.html',
        s_hostname=g.hostname,
        s_time=str(g.adc_time),
        s_time_diff=str(g.adc_time_diff),
        s_adc=str(g.adc_data),
        s_adc_mean=str(g.adc_data_mean),
        s_config=str(g.cnfg),
        s_status=str(g.status),
        s_env=str(g.env),
        s_peri=str(g.peri),
        s_about=str(about),

    )
    return s1


@bp.route('/navbar/hello')
def hello():
    s1 = render_template('/navbar/hello.html')
    return s1


@bp.route('/navbar/config', methods=['GET', 'POST'])
def config():
    """병아리, 닭 문제가 발생한다"""
    if request.method == 'POST':
        dic = request.form.to_dict()
    elif request.method == 'GET':
        dic = request.args.to_dict()
    else:
        dic = {}
        pass

    if dic:  # 폼에서 가져온 데이터가 있다면
        # print(__name__, ': new dic data found')
        # print('dic:', dic)
        lg.info('GET request, ' + str(g.cnfg))

        g.cnfg['sample']['period'] = float(dic['sample_period'])
        g.cnfg['sample']['size'] = int(dic['sample_size'])
        g.cnfg['log']['max'] = int(dic['log_max'])
        g.cnfg['proc']['method'] = dic['proc_method']

        # print('g.config:', g.config)

        '''LCD Clear Test'''

    return render_template(
        '/navbar/config.html',
        sample_period=g.cnfg['sample']['period'],
        sample_size=g.cnfg['sample']['size'],
        log_max=g.cnfg['log']['max'],
        proc_method=g.cnfg['proc']['method']
    )


@bp.route('/navbar/config-test', methods=['GET', 'POST'])
def config_test():
    """UI Control test"""
    return render_template(
        '/navbar/config-test.html',
    )


@bp.route('/navbar/miniweb', methods=['GET', 'POST'])
def miniweb():
    """mini web test"""
    return 'hello, miniweb'

    # return render_template(
    #     '/navbar/config-test.html',
    # )


@bp.route('/navbar/chart')
def chart():
    return render_template('/navbar/chart.html')


@bp.route('/chart-data')
def chart_data():
    def get_data():
        while True:
            now = datetime.now().strftime('%H:%M:%S-%f')  # OR
            # now = g.adc_time[0]
            # value = random.random()*100 # OR
            value = g.adc_data[0]
            json_data = json.dumps({'time': now, 'value': value, 'temp': g.status['cpu_temp']})
            yield f"data:{json_data}\n\n"
            time.sleep(0.2)  #

    return Response(get_data(), mimetype='text/event-stream')



# Copyright (c) 2021, 2022 Narotechnology, nash@narotechnology.com
# SPDX-License-Identifier: LicenseRef-Narotechnology-Proprietary

# 가장 먼저 실행되는 곳

print('\n\n======================')
print('<-', __name__, 'pkg init')
print('======================')

import logging
import os
import time
import socket

from flask import Flask

from flaskr import my_logging
from flaskr import my_thread3

# 플라스크 인스턴스 생성 전에 새 쓰레드 실행한다.
my_thread3.create_threads()


def create_app(test_config=None):
    """create app
    튜토리얼을 따라 입력하다
    템플릿으로 보아서는 안된다 직접 작성한 코드로 보아야 한다
    초기화 작업을 여기에 직접 넣는 것이 논리적이다
    """
    print('create_app() start')

    app = Flask(__name__, instance_relative_config=True)

    '''다음 함수는 연구가 더 필요하다
    from_file()에서 얻어지 데이터는 이 함수로 전달된다고 한다
    무슨 말인지... '''
    app.config.from_mapping(
        SECRET_KEY='dev',
        DATABASE=os.path.join(app.instance_path, 'flaskr.sqlite'),
        MY_TEST='test-21.09-nash'
    )

    # print('config:', app.config.keys())
    # logging.info(app.config.keys())

    if test_config is None:
        # load the instance config, if it exists, when not testing
        app.config.from_pyfile('config.py', silent=True)
    else:
        # load the test config if passed in
        app.config.from_mapping(test_config)

    # ensure the instance folder exists
    try:
        os.makedirs(app.instance_path)
    except OSError:
        pass

    '''
    for added function, nash
    '''
    # Added for DB
    from . import db
    db.init_app(app)

    from . import auth
    app.register_blueprint(auth.bp)

    from . import blog
    app.register_blueprint(blog.bp)
    # app.add_url_rule('/', endpoint='index')   # 루트에서 제거한다. 21.10
    app.add_url_rule('/blog', endpoint='index')

    # from . import bp_dashboard      # 오래전부터 사용되지 않는다. 22.01
    # app.register_blueprint(bp_dashboard.bp)
    # app.add_url_rule('/dashboard', endpoint='index')

    from . import bp_navbar
    app.register_blueprint(bp_navbar.bp)
    app.add_url_rule('/navbar', endpoint='index')
    # app.add_url_rule('/', endpoint='index')
    # 루트 자격을 추가한다. 21.10,
    # bp_navbar.py 에 있으므로 위 작업은 필요없다.

    print('create_app() end')
    return app



import time

from setuptools import find_packages, setup

about = {}
with open("flaskr/__about__.py") as fp:
    exec(fp.read(), about)
# later on we use: version['__version__']
print("\n\n")
print(about)

if True:
    setup(
        # name='naro-flaskr', 이미 다른 프로젝트에서 사용했다.
        name='naro-laser',
        # version='0.0.1.3',  # 21.0917
        # version='0.0.1.4',  # 21.0920
        # version='0.0.1.5',  # 21.1126
        # version='0.0.2',  # 21.1126
        # version='0.0.2.1',  # 21.1207, beta, local install test

        version=about['__version__'],
        # build_date=time.asctime(),  # NO!!!

        packages=find_packages(),
        include_package_data=True,
        zip_safe=False,
        install_requires=[
            'flask',
        ],
    )



# Copyright (c) 2021, 2022 Narotechnology, nash@narotechnology.com
# SPDX-License-Identifier: LicenseRef-Narotechnology-Proprietary

print('<-', __name__)

import socket

adc_time = [0]
adc_data = [0]
adc_time_diff = [0]
adc_data_mean = 0

hostname = socket.gethostname()

max_log = 5

'''Be careful
설정치 파일로드를 한번 실행하면 이후 이 구조의 변경은 무시된다.
'from_file' 키를 변경하는 일은 다른 모듈에서 한다.
'''
cnfg = {
    'sample': {
        'period': 0.25 * 4 * 1,  # IMPORTANT! unit is ms
        'size': 4  # buffer size
    },
    'log': {
        'max': max_log
    },
    'proc': {
        'method': 'MEAN'
    },
    'lte': {
        'dev_name': 'not defined',
        'baud_rate': 115200,
        'time_out': 1,
        'send_interval': 60 * 60,  # 1 hour
    },
    'from_file': False,
}

status = {
    'sample': {
        'mode': None,   # 'Decimation',  # or Delay
        'decimation': None,
        'delay': None
    },
    'cpu_temp': 0.0,
    'thread': {
        'lte': False,
        'led': False,

    },
    'text': '==='
}

env = {
    'temp': 0,
    'humi': 0,
    'pres': 0,
    'alti': 0
}

debug = {
    'err_count_lcd': 0
}

peri = {
    'adc': -1,  # -1 means 'not checked'
    'modem': -1,
    'lcd': -1,
    'led': -1,
    'bme280': -1,
    'dio_ext': -1,
    'button': -1
}

tele1_main = {
    'displacement': 0
}

tele2_env = {
    'temperature': 0,
    'humidity': 0,
    'pressure': 0,
    'cpu_temperature': 0
}
